% use:
%
% dE_E_par = ...    % as done in FAST_detect_locate.m
%
% ws.dE_E_fun = @dE_E_fun;
%
% ws = run_scg_fast( ws );
%
% note:
% This is a backport from onearray_run_scg_fast_logsph_c.c
%
% Guillaume Lathoud, 2011, and Kaustubh Kale
% glathoud@yahoo.fr

function [dE, E] = dE_E_fun( logsph )

% ---------- inputs

% N: most likely nLocations * 3 (for each location: azimuth, elevation, logradius)
N = length( logsph(:) );
                
% Determine whether we need to output the
% cost E as well.
compute_E = nargout > 1;

%%KK00 added this
global dE_E_par;

geometry_in = dE_E_par.geometry;
pairs_in    = dE_E_par.pairs;   
%%KK00 changed fs_in and c_in to fs and c.
fs_in       = dE_E_par.fs;
c_in        = dE_E_par.c;

nPairs      = size( pairs_in, 1 );

%%GL01 added a check. Meaning: for each row (=one pair), the first
% column  contains the index of the first microphone , and the 2nd
% column contains the index of the 2nd microphone.
if size( pairs_in, 2 ) ~= 2
  error(['dE_E_fun (onearray_logsph): size(pairs, 2) must be 2']);
end

nDimensions = size( geometry_in, 1 );
nChannels   = size( geometry_in, 2 );

w_i_in      = dE_E_par.w_i;

nFreq       = size( w_i_in(:), 1 );

theta_in    = dE_E_par.theta;

nLocations  = N / nDimensions;

% ---------- inners

% where to store the time delays
t = zeros( nPairs * nLocations, 1 );

d1 = zeros( nPairs * nLocations, 1 );
d2 = zeros( nPairs * nLocations, 1 );

dxyz1 = zeros( nPairs * nLocations * nDimensions, 1 );
dxyz2 = zeros( nPairs * nLocations * nDimensions, 1 );

D = zeros( nFreq * nPairs * nLocations, 1 );

% to save some time
if isfield( dE_E_par, 'tmp_int_array' )
  % cached
  tmp_int_array  = dE_E_par.tmp_int_array;
  tmp_int_array2 = dE_E_par.tmp_int_array2;
else
  % not cached yet
  tmp_int_array  = zeros( nPairs, 1 );
  tmp_int_array2 = zeros( nPairs, 1 );
 
  % xxx `for` loops are bad in Matlab - but for now we want something
  % that works like the MEX C does. We can optimize later.
  for a = 1:nPairs
      %%KK00 Changed pairs to pairs_in but there is a problem with pairs_in(a,1), pairs_in(a,2) as
      %%the dimensions of pairs_in are 28x2.
    tmp_int_array(a)  = (double(pairs_in(a,1)) - 1) * nDimensions; %%GL01 permit signed operation using double() cast
    tmp_int_array2(a) = (double(pairs_in(a,2)) - 1) * nDimensions; %%GL01 permit signed operation using double() cast
  end
 
  dE_E_par.tmp_int_array  = tmp_int_array;
  dE_E_par.tmp_int_array2 = tmp_int_array2;
end

% --------------------------------------------------
% Check dimensionality
% Normalize weights
% Convert coordinates from logspherical to Euclidean

if mod( N, nDimensions ) ~= 0
  error( ['dE_E_fun (onearray_logsph): inconsistent dimensions (`w`' ...
          ' and `dE_E_par.geometry`)' ])
end

weight_in = dE_E_par.weight(:);
if length( weight_in(:)) ~= nLocations
    %%KK00 added the required ]
  error( ['dE_E_fun (onearray_logsph): inconsistent dimensions' ...
          ' (`dE_E_par.weight` must have nLocations elements).' ])
end

weight_sum = sum( weight_in );

% convert logspherical coordinates to Euclidean coordinates

xyz = zeros( N, 1 );

% xxx `for` loops are bad in Matlab - but for now we want something
% that works like the MEX C does. We can optimize later.

c = 1
while c <= N

  d = c

%%KK00 changing c++ to +c.
  aa = logsph(c);  % azimuth
  c=c+1;
  bb = logsph(c);  % elevation
  c=c+1;
 
  % This is the difference with classical spherical coordinates
  rr = exp( logsph( c ) );  % radius, note the "exp"
  c=c+1; 

  %%GL01 Changed +d to an explicit d=d+1
  xyz( d ) = rr * cos( bb ) * cos( aa );  % x
  d = d+1;
  xyz( d ) = rr * cos( bb ) * sin( aa );  % y
  d = d+1;
  xyz( d ) = rr * sin( bb );              % z
  d = d+1;
 
end

% ( 5 ) Compute dxyz and time-delays

nPairs_nLocations = nPairs * nLocations;

% m: element index in dxyz1 and dxyz2 arrays
%%GL01 fixed this: indices start at 1 in MATLAB  (at 0 in C)
m = 1;
 
% xxx `for` loops are bad in Matlab - but for now we want something
% that works like the MEX C does. We can optimize later.
for c = 1:nDimensions
 
  % i: element index in d1 and d2 arrays
  %%GL01 fixed this: indices start at 1 in MATLAB  (at 0 in C)
  i = 1;
 
  %%GL01 fixed this: indices start at 1 in MATLAB  (at 0 in C)
  for b = 0:nLocations-1
 
    d = b * nDimensions;
 
    for a = 1:nPairs
   %%KK02 changed to geometry_in
      aa = xyz( c + d  ) - geometry_in( c + tmp_int_array( a )  );
      bb = xyz( c + d  ) - geometry_in( c + tmp_int_array2( a ) );
   
      dxyz1( m ) = aa;
      dxyz2( m ) = bb;
   
      % The next two lines are okay since
      % both 'd1' and 'd2' were initialized with zeroes.
      %%KK00 changed this from d1( i ) += aa * aa;
      d1( i ) =  d1( i ) + aa * aa;
      d2( i ) =  d2( i ) + bb * bb;
    %%KK00 changed these from ++
      m=m+1;
      i=i+1;
   
    end  % for a
   
  end % for b
   
end % for c


% Finish computing the Euclidean distances + time-delays
%%GL01 replaced `_val` with `_in`
cc = fs_in / c_in;

% i: element index in d1, d2 and t arrays
%%GL01 removed unnecessary line `i = 0`
for i = 1:nPairs_nLocations
   
  aa = sqrt( d1( i ) );
  bb = sqrt( d2( i ) );
 
  d1( i ) = aa;
  d2( i ) = bb;
 
  t( i ) = ( bb - aa ) * cc;

end

% ----------------------------------------------------------------------
% ( 6 ) Compute the gradient dE:
% we do it in Euclidean coordinates, then move back to logspherical coordinates

dE = zeros( size( logsph ) );

% Difference between measured and theoretical phase
% ( this is also used in the cost computation ).

%%GL01 fixed this: indices start at 1 in MATLAB  (at 0 in C)
a = 1;  % index in 'D' */
c = 1;  % index in 't' */

% xxx `for` loops are bad in Matlab - but for now we want something
% that works like the MEX C does. We can optimize later.
for n = 1:nLocations
 
  %%GL01 fixed this: indices start at 1 in MATLAB  (at 0 in C)
  b = 1; % index in 'theta'
 
  for p = 1:nPairs
 
    % Get the next time-delay in the 't' matrix
   
    tt = t( c );
    %%KK00 changed this from ++
    c=c+1;
   
    for i = 1:nFreq
      %%KK02 changed to theta_in and w_i_in
      D( a ) = theta_in( b ) - w_i_in( i ) * tt;
      %%KK00 changed this from ++
    a=a+1;
    b=b+1;
    end
 
  end
 
end

% Now compute each component of the gradient dE
%%GL01 replaced `_val` with `_in`
aa = - fs_in / ( 2 * c_in * nPairs * nFreq * weight_sum );

% index in 'xyz'
%%GL01 fixed this: indices start at 1 in MATLAB  (at 0 in C)
a = 1; % x
b = 2; % y
c = 3; % z

%%GL01 fixed this: indices start at 1 in MATLAB  (at 0 in C)
t = 1;   % index in 'd1' and 'd2', as well as x in 'dxyz1' and 'dxyz2'
u = 1 +     nPairs_nLocations;       % index of y in 'dxyz1' and 'dxyz2'
v = 1 + 2 * nPairs_nLocations;       % index of z in 'dxyz1' and 'dxyz2'

% index in D
%%GL01 fixed this: indices start at 1 in MATLAB  (at 0 in C)
d = 1;

% xxx `for` loops are bad in Matlab - but for now we want something
% that works like the MEX C does. We can optimize later.
for k = 1:nLocations
 
  % Where we'll temporarily store the gradient
  % in Euclidean coordinates, for location #k.
 
  dE_x = 0;
  dE_y = 0;
  dE_z = 0;
 
  for p = 1:nPairs
 
    % Compute the factor in sine, which is common to x,y,z
 
    cc = 0;
    for i = 1:nFreq
    %%KK00 changed this from += and ++
    %%KK02 changed to w_i_in
      cc = cc + w_i_in( i ) * sin( D( d ) );
  d= d+1;
    end
 
    % Update the sum over pairs, for each coordinate x,y,z
 
    dd = cc / d2( t );
    ee = cc / d1( t );
  %%KK00 changed this from +=
    dE_x = dE_x + dxyz2( t ) * dd - dxyz1( t ) * ee;        % x
    dE_y = dE_y + dxyz2( u ) * dd - dxyz1( u ) * ee;        % y
    dE_z = dE_z + dxyz2( v ) * dd - dxyz1( v ) * ee;        % z
 
    % move to next element in 'd1', 'd2', 'dxyz1', 'dxyz2'
    %%KK00 changed this from ++
    t=t+1;
    u=u+1;
    v=v+1;
 
  end % for p

  % Scaling factor
 %%KK02 changed to weight_in
  bb = aa * weight_in( k );
 %%KK00 changed this from *=
  dE_x = dE_x*bb;
  dE_y = dE_y*bb;
  dE_z = dE_z*bb;
 
  % Finally translate back to logspherical coordinates
 
  xx = xyz( a ); % x
  yy = xyz( b ); % y
  zz = xyz( c ); % z
 
  tt = logsph( a ); % azimuth 
  uu = logsph( b ); % elevation
  vv = logsph( c ); % log radius
 
 
  dE( a ) = -yy               * dE_x + xx        * dE_y;
  dE( b ) = -zz * ( cos( tt ) * dE_x + sin( tt ) * dE_y ) + vv * cos( uu ) * dE_z;
 
  % Note: compared to classical spherical coordinates,
  % only this line differs
  % It is the definition of ( dE / ds ) where s = log( radius )
  dE( c ) = xx                * dE_x + yy        * dE_y     + zz           * dE_z;
 
 
  % Move to the next location in 'xyz', 'logsph' and 'dE'
  %%KK00 changed this from ++
  %%GL01 fixed the a,b,c update
  a = a + 3;
  b = b + 3;
  c = c + 3;
 
end % for k


% ----------------------------------------------------------------------
% ( 7 ) If relevant, compute the cost E

if compute_E
 
  % Where we'll store the partial sum
  aa = 0;
 
  % Index in 'D'
  %%GL01 fixed this: indices start at 1 in MATLAB  (at 0 in C)
  a = 1;
 
  for n = 1:nLocations

    bb = 0;
 
    q = nPairs * nFreq;
    for p = 1:q
        %%KK00 changed this from += and ++ for a and [ ] for cos.
        bb = bb + cos( D(a) );
            a=a+1;
    end

    % scale with the weight of this location,
    % and update the partial sum.
    %%KK00 changed this from +=
    %%KK02 changed to weight_in
    aa = aa + bb * weight_in( n );
 
  end
 
  E = 0.5 * ( 1 - aa / ( nFreq * nPairs * weight_sum ) );

end
